@@ -55,7 +55,7 @@ sqlite3 *db;
 #if PLAINGTK == 1
 char default_db[] = "fuelpad.db";
 #else
-char default_db[] = "/home/user/MyDocs/fuelpad.db";
+char default_db[] = DATABASE_FILENAME;
 #endif
 char *result_db;
 sqlite3_stmt *ppStmtSaveRecord;
@@ -73,6 +73,8 @@ sqlite3_stmt *ppStmtRecords;
 sqlite3_stmt *ppStmtExport;
 sqlite3_stmt *ppStmtExportCar;
 sqlite3_stmt *ppStmtAddRecord;
+sqlite3_stmt *ppStmtNextFull;
+sqlite3_stmt *ppStmtPrevFull;
 sqlite3_stmt *ppStmtDeleteRecord;
 sqlite3_stmt *ppStmtUpdateRecord;
 sqlite3_stmt *ppStmtGetYears;
@@ -370,7 +372,7 @@ void db_connect(void)
 
       /* prepare sql query for obtaining one record */
       rc = sqlite3_prepare_v2(db, 
-"SELECT day,km,trip,fill,price,service,oil,tires,notes,id "
+"SELECT day,km,trip,fill,price,service,oil,tires,notes,consum,id "
 "FROM record WHERE id=?;", -1, &ppStmtOneRecord, NULL);
 
       /* prepare sql query for obtaining all the records */
@@ -394,6 +396,18 @@ void db_connect(void)
 "INTO record(carid,driverid,day,km,trip,fill,consum,price,priceperlitre,service,oil,tires,notes) "
 "VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?);", -1, &ppStmtAddRecord, NULL);
 
+      /* prepare sql query for calculating the full fill (fill and total km) */
+      rc = sqlite3_prepare_v2(db, 
+"SELECT "
+"km,trip,fill,consum,id FROM record WHERE carid=? AND km>? AND fill>0 AND consum>0 ORDER BY km LIMIT 1;", 
+			      -1, &ppStmtNextFull, NULL);
+
+      /* prepare sql query for calculating the full fill (fill and total km) */
+      rc = sqlite3_prepare_v2(db, 
+"SELECT "
+"km,trip,fill,consum,id FROM record WHERE carid=? AND km<? AND fill>0 ORDER BY km DESC;", 
+			      -1, &ppStmtPrevFull, NULL);
+
       /* prepare sql query for changing a record */
       rc = sqlite3_prepare_v2(db, 
 "UPDATE record "
@@ -404,27 +418,6 @@ void db_connect(void)
 "SELECT DISTINCT STRFTIME('%Y',date(day)) FROM record WHERE carid=?;", -1,
 			      &ppStmtGetYears, NULL);
 
-      /* Prepare sql query for obtaining the report data */
-      rc = sqlite3_prepare_v2(db,
-"SELECT STRFTIME('%Y',day),MIN(km),MAX(km),MAX(km)-MIN(km),"
-"SUM(fill),SUM(price),SUM(fill)/SUM(trip)*100,SUM(price)/SUM(fill),"
-"SUM(price)/SUM(trip),SUM(oil),SUM(oil)/(MAX(km)-MIN(km)),"
-"SUM(service),SUM(service)/(MAX(km)-MIN(km)),SUM(tires),"
-"SUM(tires)/(MAX(km)-MIN(km)) "
-"FROM record WHERE carid=? AND "
-/* "STRFTIME('%Y',DATE(day))>= '?' AND " */
-/* "STRFTIME('%Y',DATE(day))<= '?' AND " */
-"km NOT IN (0) "
-"GROUP BY STRFTIME('%Y',DATE(day));", -1, &ppStmtGetReport, NULL);
-/* "SELECT STRFTIME('%Y',day),MIN(km),MAX(km),MAX(km)-MIN(km)," */
-/* "SUM(fill),SUM(price),SUM(fill)/SUM(trip)*100,SUM(price)/SUM(fill)," */
-/* "SUM(price)/SUM(trip),SUM(oil),SUM(oil)/(MAX(km)-MIN(km))," */
-/* "SUM(service),SUM(service)/(MAX(km)-MIN(km)),SUM(tires)," */
-/* "SUM(tires)/(MAX(km)-MIN(km)) " */
-/* "FROM record WHERE carid=? AND STRFTIME('%Y',DATE(day))>='?' AND " */
-/* "STRFTIME('%Y',DATE(day))<='?' AND km NOT IN (0) " */
-/* "GROUP BY STRFTIME('%Y',DATE(day));", -1, &ppStmtGetReport, NULL); */
-
       /* prepare sql query for adding an alarmtype */
       rc = sqlite3_prepare_v2(db,
 "INSERT "
@@ -502,6 +495,8 @@ void db_close(void)
   sqlite3_finalize(ppStmtExport);
   sqlite3_finalize(ppStmtExportCar);
   sqlite3_finalize(ppStmtAddRecord);
+  sqlite3_finalize(ppStmtNextFull);
+  sqlite3_finalize(ppStmtPrevFull);
   sqlite3_finalize(ppStmtUpdateRecord);
   sqlite3_finalize(ppStmtGetYears);
   sqlite3_finalize(ppStmtGetReport);
@@ -581,7 +576,7 @@ float get_emission_per_litre(int fueltype)
  * \return CO2 emission in g/km
  *
  */
-float calc_co2_emission(float fill, float trip, int carid)
+float calc_co2_emission(float consum, int carid)
 {
   float result;
   int fueltype;
@@ -590,8 +585,8 @@ float calc_co2_emission(float fill, float trip, int carid)
   if (fueltype<0) fueltype=0;
   if (fueltype>1) fueltype=1;
 
-  if (trip>1e-5)
-    result=get_emission_per_litre(fueltype)*fill/trip;
+  if (consum>1e-5)
+    result=get_emission_per_litre(fueltype)*consum/100.0;
   else
     result=0.0;
   return result;
@@ -779,11 +774,12 @@ float db_get_last_km(void)
 }
 
 /**
- * \fn float db_get_last_refill(void)
+ * \fn float db_get_last_refill(double newkm)
  * \brief Finds the km of last refill
+ * \param newkm the km of the new record
  * \return Last refill km in SI units
  */
-float db_get_last_refill(void)
+float db_get_last_refill(double newkm)
 {
   char **sqlresult;
   char *sqlquery;
@@ -794,7 +790,7 @@ float db_get_last_refill(void)
 
   ret=0.0;
  if (db) {
-   sqlquery = sqlite3_mprintf("SELECT km FROM record WHERE carid=%d AND trip>0 ORDER BY km DESC LIMIT 1;",currentcar);
+   sqlquery = sqlite3_mprintf("SELECT km FROM record WHERE carid=%d AND trip>0 AND km<%f ORDER BY km DESC LIMIT 1;",currentcar,newkm);
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
 /*    sqlite3_free((void *)sqlquery); */
    sqlite3_free(sqlquery);
@@ -810,6 +806,94 @@ float db_get_last_refill(void)
  return ret;
 }
 
+void db_find_prev_full(double km, double *fullfill, double *fullkm)
+{
+/*   char **sqlresult; */
+/*   char *sqlquery; */
+/*   int sqlrows; */
+/*   int sqlcolumns; */
+/*   int rc; */
+  double fill;
+  double trip;
+  double consum;
+  
+  *fullfill=0.0;
+  *fullkm=0.0;
+  if (db) {
+    if (!(SQLITE_OK != sqlite3_bind_int(ppStmtPrevFull, 1, currentcar) /* carid */
+	  || SQLITE_OK != sqlite3_bind_double(ppStmtPrevFull, 2, km)) ) {
+	  while (SQLITE_ROW == sqlite3_step(ppStmtPrevFull)) { /* full tank record succesfully found */
+	    trip=sqlite3_column_double(ppStmtPrevFull,1);
+	    fill=sqlite3_column_double(ppStmtPrevFull,2);
+	    consum=sqlite3_column_double(ppStmtPrevFull,3);
+	    if (abs(consum)>1e-3) /* Full fill found*/
+	      break;
+	    else {
+	      *fullfill += fill;
+	      *fullkm += trip;
+	    }
+	
+	  }
+	  sqlite3_reset(ppStmtPrevFull);
+	}
+  }
+}
+
+sqlite_int64 db_find_next_full(double km, double *fullfill, double *fullkm)
+{
+  sqlite_int64 id;
+  double fill;
+  double trip;
+
+  id=-1;
+  if (db) {
+    if (!(SQLITE_OK != sqlite3_bind_int(ppStmtNextFull, 1, currentcar) /* carid */
+	  || SQLITE_OK != sqlite3_bind_double(ppStmtNextFull, 2, km))) {
+      if(SQLITE_ROW == sqlite3_step(ppStmtNextFull)) {
+	km = sqlite3_column_double(ppStmtNextFull,0);
+	trip = sqlite3_column_double(ppStmtNextFull,1);
+	fill = sqlite3_column_double(ppStmtNextFull,2);
+	id = sqlite3_column_int64(ppStmtNextFull,4);
+	sqlite3_reset(ppStmtNextFull);
+	db_find_prev_full(km,fullfill,fullkm);
+	*fullfill += fill;
+	*fullkm += trip;
+      }
+      else
+	sqlite3_reset(ppStmtNextFull);
+    }
+  }
+  return id;
+}
+
+int db_create_fillview(void)
+{
+  char **sqlresult;
+  char *sqlquery;
+  int sqlrows;
+  int sqlcolumns;
+  int rc;
+
+  sqlquery = sqlite3_mprintf("CREATE TEMP VIEW fillview AS SELECT * FROM record WHERE carid=%d ORDER BY km LIMIT -1 OFFSET 1;",currentcar);
+  rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
+  sqlite3_free((void *)sqlquery);
+  sqlite3_free_table(sqlresult);
+
+  return rc;
+}
+
+void db_drop_fillview(void)
+{
+  char **sqlresult;
+  int sqlrows;
+  int sqlcolumns;
+  int rc;
+
+  rc = sqlite3_get_table(db, "DROP VIEW fillview;", &sqlresult, &sqlrows, &sqlcolumns, NULL);
+  if (rc==SQLITE_OK)
+    sqlite3_free_table(sqlresult);
+}
+
 /**
  * \fn int db_get_monthly_data(int year, double **x, double **fill, double **trip, double **consum, double **priceperlitre)
  * \brief Get the monthly data of a given year
@@ -840,10 +924,7 @@ int db_get_monthly_data(int year, double **x, double **fill, double **trip, doub
   *consum=NULL;
   *priceperlitre=NULL;
   if (db) {
-    sqlquery = sqlite3_mprintf("CREATE TEMP VIEW fillview AS SELECT * FROM record WHERE carid=%d ORDER BY km LIMIT -1 OFFSET 1;",currentcar);
-    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
-    sqlite3_free((void *)sqlquery);
-    sqlite3_free_table(sqlresult);
+    rc = db_create_fillview();
     if (rc==SQLITE_OK) {
       // select * from fillview where day between '2007-01-01' and '2007-12-31';
       // SELECT STRFTIME(\"%m\",DATE(day)),STRFTIME(\"%Y\",DATE(day)),SUM(fill),SUM(trip),SUM(fill)/SUM(trip)*100 FROM fillview WHERE carid=2 AND STRFTIME(\"%Y\",DATE(day))=='2007' GROUP BY STRFTIME(\"%m\",day);
@@ -870,9 +951,7 @@ int db_get_monthly_data(int year, double **x, double **fill, double **trip, doub
 	  PDEBUG("No data found\n");
 	sqlite3_free_table(sqlresult);
 
-	rc = sqlite3_get_table(db, "DROP VIEW fillview;", &sqlresult, &sqlrows, &sqlcolumns, NULL);
-	if (rc==SQLITE_OK)
-	  sqlite3_free_table(sqlresult);
+	db_drop_fillview();
       } // if(rc==SQLITE_OK)
       else
 	PDEBUG("No data found\n");
@@ -885,12 +964,18 @@ int db_get_monthly_data(int year, double **x, double **fill, double **trip, doub
 }
 
 /**
- * \fn float db_get_totalkm(void)
+ * \fn float db_get_totalkm(enum dbtimespan timespan)
+ * \param timespan either OVERALL, LASTMONTH or LASTYEAR
  * \brief Get the total km of currentcar
  * \return Total km
+ *
+ * Return the total km for the whole data in the database or for last
+ * month or last year.
+ *
  */
-float db_get_totalkm(void)
+float db_get_totalkm(enum dbtimespan timespan)
 {
+ static char querystr[]="SELECT %s FROM record WHERE carid=%d%s";
  char **sqlresult;
  char *sqlquery;
  int sqlrows;
@@ -900,7 +985,20 @@ float db_get_totalkm(void)
 
  ret=0.0;
  if (db) {
-   sqlquery = sqlite3_mprintf("SELECT max(km)-min(km) FROM record WHERE carid=%d;",currentcar);
+   switch (timespan) {
+   case OVERALL:
+   case SPANEND:
+     sqlquery = sqlite3_mprintf(querystr,"MAX(km)-MIN(km)",currentcar,";");
+     break;
+   case LASTMONTH:
+     sqlquery = sqlite3_mprintf(querystr,"SUM(trip)",currentcar," AND day BETWEEN DATE('now','-1 month') AND DATE('now');");
+     break;
+   case LASTYEAR:
+     sqlquery = sqlite3_mprintf(querystr,"SUM(trip)",currentcar," AND day BETWEEN DATE('now','-1 year') AND DATE('now');");
+     break;
+   default:
+    sqlquery = sqlite3_mprintf(querystr,"MAX(km)-MIN(km)",currentcar,";");
+   }
    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
 /*    sqlite3_free((void *)sqlquery); */
    sqlite3_free(sqlquery);
@@ -916,6 +1014,61 @@ float db_get_totalkm(void)
  return ret;
 }
 
+/**
+ * \fn float db_get_totalfillkm(void)
+ * \param timespan either OVERALL, LASTMONTH or LASTYEAR
+ * \brief Get the total km of currentcar for average fuel consumption calculation
+ * \return Total km
+ *
+ * Return the total km for the whole data in the database or for last
+ * month or last year. This takes into account only those records with fill>0.
+ *
+ */
+float db_get_totalfillkm(enum dbtimespan timespan)
+{
+ static char querystr[]="SELECT sum(trip) FROM fillview WHERE carid=%d AND fill>0%s";
+ char **sqlresult;
+ char *sqlquery;
+ int sqlrows;
+ int sqlcolumns;
+ int rc;
+ float ret;
+
+ ret=0.0;
+ if (db) {
+    rc = db_create_fillview();
+    if (rc==SQLITE_OK) {
+      switch (timespan) {
+      case OVERALL:
+      case SPANEND:
+	sqlquery = sqlite3_mprintf(querystr,currentcar,";");
+	break;
+      case LASTMONTH:
+	sqlquery = sqlite3_mprintf(querystr,currentcar," AND day BETWEEN DATE('now','-1 month') AND DATE('now');");
+	break;
+      case LASTYEAR:
+	sqlquery = sqlite3_mprintf(querystr,currentcar," AND day BETWEEN DATE('now','-1 year') AND DATE('now');");
+	break;
+      default:
+	sqlquery = sqlite3_mprintf(querystr,currentcar,";");
+      }
+      rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
+      /*    sqlite3_free((void *)sqlquery); */
+      sqlite3_free(sqlquery);
+      if (rc==SQLITE_OK) {
+	if (sqlcolumns*(sqlrows+1)>0) {
+	  if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
+	    ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
+	}
+	sqlite3_free_table(sqlresult);
+      }
+      db_drop_fillview();
+   }
+ }
+
+ return ret;
+}
+
 /**
  * \fn float db_get_totalcost(void)
  * \brief Get the total cost of currentcar
@@ -962,8 +1115,9 @@ float db_get_totalcost(void)
  * first one?
  *
  */
-float db_get_totalfill(void)
+float db_get_totalfill(enum dbtimespan timespan)
 {
+ static char querystr[]="SELECT SUM(fill) FROM fillview WHERE carid=%d%s";
  char **sqlresult;
  char *sqlquery;
  int sqlrows;
@@ -973,58 +1127,34 @@ float db_get_totalfill(void)
 
  ret=0.0;
  if (db) {
-/*    sqlquery = sqlite3_mprintf("CREATE TEMP VIEW fillview AS SELECT fill FROM record WHERE carid=%d ORDER BY km LIMIT -1 OFFSET 1;",currentcar); */
-/*    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL); */
-/*    sqlite3_free((void *)sqlquery); */
-/*    sqlite3_free_table(sqlresult); */
-/*    if (rc==SQLITE_OK) { */
-/*      printf("succesfully created temporary view\n"); */
-
-/*      rc = sqlite3_get_table(db, "SELECT sum(fill) FROM fillview;", &sqlresult, &sqlrows, &sqlcolumns, NULL); */
-/*      printf("sqlrows=%d, sqlcolumns=%d\n",sqlrows,sqlcolumns); */
-/*      if (rc==SQLITE_OK) { */
-/*        printf("succesfully queried from temporary view\n"); */
-/*        if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL) */
-/* 	 ret=atof(sqlresult[sqlcolumns*(sqlrows+1)-1]); */
-/*        else */
-/* 	 ret=0.0; */
-/*        printf("ret=%f\n",ret); */
-/*        sqlite3_free_table(sqlresult); */
-
-/*        rc = sqlite3_get_table(db, "DROP VIEW fillview;", &sqlresult, &sqlrows, &sqlcolumns, NULL); */
-/*      if (rc==SQLITE_OK) */
-/*        printf("succesfully dropped temporary view\n"); */
-/*      sqlite3_free_table(sqlresult); */
-
-/*      } */
-/*      else */
-/*        ret=0.0; */
-/* /\*      sqlite3_free((void *)sqlquery); *\/ */
-/*    } */
-
-   sqlquery = sqlite3_mprintf("SELECT sum(fill) FROM record WHERE carid=%d",currentcar);
-   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
-   sqlite3_free((void *)sqlquery);
-   if (rc==SQLITE_OK) {
-     if (sqlcolumns*(sqlrows+1)>0) {
-       if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL) {
-	 ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
-	 sqlite3_free_table(sqlresult);
-	 sqlquery = sqlite3_mprintf("SELECT fill FROM record WHERE carid=%d ORDER BY km LIMIT 1",currentcar);
-	 rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
-	 sqlite3_free((void *)sqlquery);
-	 if (rc==SQLITE_OK) {
-	   if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
-	     ret -= atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
-	 }
-	 sqlite3_free_table(sqlresult);
-       }
-     }
-     else {
-       sqlite3_free_table(sqlresult);
-     }
+    rc = db_create_fillview();
+    if (rc==SQLITE_OK) {
+      switch (timespan) {
+      case OVERALL:
+      case SPANEND:
+	sqlquery = sqlite3_mprintf(querystr,currentcar,";");
+	break;
+      case LASTMONTH:
+	sqlquery = sqlite3_mprintf(querystr,currentcar," AND day BETWEEN DATE('now','-1 month') AND DATE('now');");
+	break;
+      case LASTYEAR:
+	sqlquery = sqlite3_mprintf(querystr,currentcar," AND day BETWEEN DATE('now','-1 year') AND DATE('now');");
+	break;
+      default:
+	sqlquery = sqlite3_mprintf(querystr,currentcar,";");
+      }
+      rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
+      /*    sqlite3_free((void *)sqlquery); */
+      sqlite3_free(sqlquery);
+      if (rc==SQLITE_OK) {
+	if (sqlcolumns*(sqlrows+1)>0) {
+	  if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
+	    ret=atof_l(sqlresult[sqlcolumns*(sqlrows+1)-1],"C");
+	}
+	sqlite3_free_table(sqlresult);
+      }
+      db_drop_fillview();
    }
-
  }
 
  return ret;
@@ -1059,12 +1189,10 @@ int db_get_yearly_consum(int **year, int **numfills, float **minconsum, float **
   *avgconsum=NULL;
   *maxconsum=NULL;
   if (db) {
-    sqlquery = sqlite3_mprintf("CREATE TEMP VIEW fillview AS SELECT * FROM record WHERE carid=%d ORDER BY km LIMIT -1 OFFSET 1;",currentcar);
-    rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
-    sqlite3_free((void *)sqlquery);
-    sqlite3_free_table(sqlresult);
+    rc = db_create_fillview();
     if (rc==SQLITE_OK) {
-      sqlquery = sqlite3_mprintf("SELECT STRFTIME(\"%s\",DATE(day)),COUNT(*),MIN(consum),SUM(fill)/SUM(trip)*100,MAX(consum) FROM fillview WHERE fill>0 GROUP BY STRFTIME(\"%s\",DATE(day))","%Y","%Y");
+      /* consum limit 1e5 cuts the Inf values out */
+      sqlquery = sqlite3_mprintf("SELECT STRFTIME(\"%s\",DATE(day)),COUNT(*),MIN(consum),SUM(fill)/SUM(trip)*100,MAX(consum) FROM fillview WHERE consum>0 AND consum<1e5 GROUP BY STRFTIME(\"%s\",DATE(day))","%Y","%Y");
       rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
       sqlite3_free((void *)sqlquery);
       if (rc==SQLITE_OK) {
@@ -1089,9 +1217,7 @@ int db_get_yearly_consum(int **year, int **numfills, float **minconsum, float **
 
       sqlite3_free_table(sqlresult);
 
-      rc = sqlite3_get_table(db, "DROP VIEW fillview;", &sqlresult, &sqlrows, &sqlcolumns, NULL);
-      if (rc==SQLITE_OK)
-	sqlite3_free_table(sqlresult);
+      db_drop_fillview();
     } // if(rc==SQLITE_OK)
 
   } // if (db)
@@ -1119,7 +1245,7 @@ int db_get_consum_bin(int year, float minconsum, float maxconsum)
 
   ret=0;
   if (db) {
-    sqlquery = sqlite3_mprintf("SELECT COUNT(*) FROM record WHERE carid=%d AND STRFTIME(\"%s\",DATE(day))='%d' AND fill > 0 AND consum BETWEEN %f AND %f",currentcar,"%Y",year,minconsum,maxconsum);
+    sqlquery = sqlite3_mprintf("SELECT COUNT(*) FROM record WHERE carid=%d AND STRFTIME(\"%s\",DATE(day))='%d' AND consum > 0 AND consum BETWEEN %f AND %f",currentcar,"%Y",year,minconsum,maxconsum);
     rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
     sqlite3_free((void *)sqlquery);
     if (rc==SQLITE_OK) {
@@ -1218,6 +1344,41 @@ void db_get_current_driver(char **driver)
   db_get_driver(driver, currentdriver);
 }
 
+/**
+ * \fn void sqlite_int64 db_get_driverid(int index)
+ * \brief Get the database id of a driver
+ * \param index index in the database
+ *
+ * This function returns carid of a driver in the database. The parameter
+ * index is the ordinal of the driver.
+ *
+ */
+sqlite_int64 db_get_driverid(int index)
+{
+ char **sqlresult;
+ char *sqlquery;
+ int sqlrows;
+ int sqlcolumns;
+ int rc;
+ sqlite_int64 ret;
+
+ if (db) {
+   sqlquery = sqlite3_mprintf("SELECT id FROM driver LIMIT 1 OFFSET %d",index);
+   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
+   if (rc==SQLITE_OK) {
+     ret=atol(sqlresult[sqlcolumns*(sqlrows+1)-1]);
+     sqlite3_free_table(sqlresult);
+   }
+   else
+     ret=0;
+ }
+ else
+   ret=0;
+
+ return ret;
+}
+
+
 /**
  * \fn int db_get_numof_drivers(void)
  * \brief Get the number of drivers in the database
@@ -1604,6 +1765,40 @@ void db_get_current_car(char **car)
 /*    strcpy(car,"db closed"); */
 }
 
+/**
+ * \fn void sqlite_int64 db_get_carid(int index)
+ * \brief Get the database id of a car
+ * \param index index in the database
+ *
+ * This function returns carid of a car in the database. The parameter
+ * index is the ordinal of the car.
+ *
+ */
+sqlite_int64 db_get_carid(int index)
+{
+ char **sqlresult;
+ char *sqlquery;
+ int sqlrows;
+ int sqlcolumns;
+ int rc;
+ sqlite_int64 ret;
+
+ if (db) {
+   sqlquery = sqlite3_mprintf("SELECT id FROM car LIMIT 1 OFFSET %d",index);
+   rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
+   if (rc==SQLITE_OK) {
+     ret=atol(sqlresult[sqlcolumns*(sqlrows+1)-1]);
+     sqlite3_free_table(sqlresult);
+   }
+   else
+     ret=0;
+ }
+ else
+   ret=0;
+
+ return ret;
+}
+
 /**
  * \fn int db_estimate_date(float km, char **estdate, char *lastdate, double interval)
  * \brief Estimate the date when the given km will be reached
@@ -1694,12 +1889,13 @@ float db_estimate_km(char *lastdate, double interval)
     sqlite3_free((void *)sqlquery);
     sqlite3_free_table(sqlresult);
     if (rc==SQLITE_OK) {
-      if (lastdate != NULL) {
+      if (lastdate != NULL && interval > 0.0) {
 	sqlquery = sqlite3_mprintf("SELECT MIN(km)+(MAX(km)-MIN(km))/(STRFTIME(\"%s\",DATE(MAX(day)))-STRFTIME(\"%s\",DATE(MIN(day))))*(STRFTIME(\"%s\",DATE(\"%s\",'+%d months'))-STRFTIME(\"%s\",DATE(MIN(day)))) FROM kmview;","%s","%s","%s",lastdate,(int)interval,"%s");
 	rc = sqlite3_get_table(db, sqlquery, &sqlresult, &sqlrows, &sqlcolumns, NULL);
 	sqlite3_free((void *)sqlquery);
 	if (rc==SQLITE_OK) {
-	  ret=atof(sqlresult[sqlcolumns*(sqlrows+1)-1]);
+	  if (sqlresult[sqlcolumns*(sqlrows+1)-1] != NULL)
+	    ret=atof(sqlresult[sqlcolumns*(sqlrows+1)-1]);
 	  sqlite3_free_table(sqlresult);
 	}
       }
@@ -1900,6 +2096,228 @@ int db_delete_reminder(int id)
  return retval;
 }
 
+int db_init_report_query(unsigned int carid)
+{
+  int rc;
+
+  rc = db_create_fillview();
+  /* Prepare sql query for obtaining the report data */
+  rc = sqlite3_prepare_v2(db,
+"SELECT STRFTIME('%Y',day),MIN(km),MAX(km),MAX(km)-MIN(km),"
+"SUM(fill),SUM(price),SUM(fill)/SUM(trip)*100,SUM(price)/SUM(fill),"
+"SUM(price)/SUM(trip),SUM(oil),SUM(oil)/(MAX(km)-MIN(km)),"
+"SUM(service),SUM(service)/(MAX(km)-MIN(km)),SUM(tires),"
+"SUM(tires)/(MAX(km)-MIN(km)) "
+"FROM fillview WHERE carid=? AND "
+"km NOT IN (0) "
+"GROUP BY STRFTIME('%Y',DATE(day)) "
+"UNION "
+"SELECT 'Total',MIN(km),MAX(km),MAX(km)-MIN(km),"
+"SUM(fill),SUM(price),SUM(fill)/SUM(trip)*100,SUM(price)/SUM(fill),"
+"SUM(price)/SUM(trip),SUM(oil),SUM(oil)/(MAX(km)-MIN(km)),"
+"SUM(service),SUM(service)/(MAX(km)-MIN(km)),SUM(tires),"
+"SUM(tires)/(MAX(km)-MIN(km)) "
+"FROM fillview WHERE carid=? AND "
+"km NOT IN (0) ", -1, &ppStmtGetReport, NULL);
+  
+  if (rc==SQLITE_OK) {
+    rc=sqlite3_bind_int(ppStmtGetReport, 1, carid);
+    rc=sqlite3_bind_int(ppStmtGetReport, 2, carid);
+  }
+
+  return rc;
+}
+
+void db_reset_report_query(void)
+{
+  sqlite3_reset(ppStmtGetReport);
+}
+
+
+int db_step_report_query(void)
+{
+  return sqlite3_step(ppStmtGetReport);
+}
+
+const unsigned char *db_report_column_text(int col)
+{
+  return sqlite3_column_text(ppStmtGetReport,col);
+}
+
+double db_report_column_double(int col)
+{
+  return sqlite3_column_double(ppStmtGetReport,col);
+}
+
+void db_close_report_query(void)
+{
+  db_drop_fillview();
+}
+
+sqlite_int64 db_add_record(char * date,
+			   double km,
+			   double trip,
+			   double fill,
+			   double consum,
+			   double price,
+			   double priceperlitre,
+			   double service,
+			   double oil,
+			   double tires,
+			   char *notes)
+{
+  sqlite_int64 ret;
+
+  ret=-1;
+
+  if (db) {
+
+    	if (SQLITE_OK != sqlite3_bind_int(ppStmtAddRecord, 1, currentcar) /* carid */
+	    || SQLITE_OK != sqlite3_bind_int(ppStmtAddRecord, 2, currentdriver) /* driverid */
+	    || SQLITE_OK != sqlite3_bind_text(ppStmtAddRecord, 3, 
+					      date,
+					      -1, SQLITE_STATIC)
+	    || SQLITE_OK != sqlite3_bind_double(ppStmtAddRecord, 4,
+						km)
+	    || SQLITE_OK != sqlite3_bind_double(ppStmtAddRecord, 5,
+						trip)
+	    || SQLITE_OK != sqlite3_bind_double(ppStmtAddRecord, 6,
+						fill)
+	    || SQLITE_OK != sqlite3_bind_double(ppStmtAddRecord, 7,
+						consum)
+	    || SQLITE_OK != sqlite3_bind_double(ppStmtAddRecord, 8,
+						price)
+	    || SQLITE_OK != sqlite3_bind_double(ppStmtAddRecord, 9,
+						priceperlitre)
+	    || SQLITE_OK != sqlite3_bind_double(ppStmtAddRecord, 10,
+						service)
+	    || SQLITE_OK != sqlite3_bind_double(ppStmtAddRecord, 11,
+						oil)
+	    || SQLITE_OK != sqlite3_bind_double(ppStmtAddRecord, 12,
+						tires)
+	    || SQLITE_OK != sqlite3_bind_text(ppStmtAddRecord, 13, 
+					      notes,
+					      -1, SQLITE_STATIC)
+	    || SQLITE_DONE != sqlite3_step(ppStmtAddRecord)) {
+	  PDEBUG("Problem saving result");
+	}
+	else
+	  ret = db_get_last_insert_rowid();
+	sqlite3_reset(ppStmtAddRecord);
+  }
+
+  return ret;
+}
+
+sqlite_int64 db_update_record(sqlite_int64 id,
+			      char * date,
+			      double km,
+			      double trip,
+			      double fill,
+			      double consum,
+			      double price,
+			      double priceperlitre,
+			      double service,
+			      double oil,
+			      double tires,
+			      char *notes)
+{
+  sqlite_int64 ret;
+
+  ret=-1;
+
+  if (db) {
+
+      if (SQLITE_OK == sqlite3_bind_int(ppStmtOneRecord, 1, id)) {
+	if (SQLITE_ROW == sqlite3_step(ppStmtOneRecord)) {
+
+
+	  if ( 
+	      (  SQLITE_OK != sqlite3_bind_text(ppStmtUpdateRecord, 1,
+						(date == NULL) ? 
+						sqlite3_column_text(ppStmtOneRecord,0) 
+						: date,
+						-1, SQLITE_STATIC) )
+	      || ( SQLITE_OK != sqlite3_bind_double(ppStmtUpdateRecord, 2,
+						    (km < 0.0) ?
+						    sqlite3_column_double(ppStmtOneRecord,1) 
+						    : km) )
+	      || ( SQLITE_OK != sqlite3_bind_double(ppStmtUpdateRecord, 3,
+						    (trip < 0.0) ?
+						    sqlite3_column_double(ppStmtOneRecord,2) 
+						    : trip) )
+	      || ( SQLITE_OK != sqlite3_bind_double(ppStmtUpdateRecord, 4,
+						    (fill < 0.0) ?
+						    sqlite3_column_double(ppStmtOneRecord,3) 
+						    : fill) )
+	      || ( SQLITE_OK != sqlite3_bind_double(ppStmtUpdateRecord, 5,
+						    (consum < 0.0) ?
+						    sqlite3_column_double(ppStmtOneRecord,9)
+						    : consum) )
+	      || ( SQLITE_OK != sqlite3_bind_double(ppStmtUpdateRecord, 6,
+						    (price < 0.0) ?
+						    sqlite3_column_double(ppStmtOneRecord,4)
+						    : price) )
+	      || ( SQLITE_OK != sqlite3_bind_double(ppStmtUpdateRecord, 7,
+						    (priceperlitre < 0.0) ? 
+						    sqlite3_column_double(ppStmtOneRecord,4)/
+						    sqlite3_column_double(ppStmtOneRecord,3)
+						    : priceperlitre) )
+	      || (  SQLITE_OK != sqlite3_bind_double(ppStmtUpdateRecord, 8,
+						     (service < 0.0) ?
+						     sqlite3_column_double(ppStmtOneRecord,5)
+						     : service) )
+	      || ( SQLITE_OK != sqlite3_bind_double(ppStmtUpdateRecord, 9,
+						    (oil < 0.0) ?
+						    sqlite3_column_double(ppStmtOneRecord,6)
+						    : oil) )
+	      || ( SQLITE_OK != sqlite3_bind_double(ppStmtUpdateRecord, 10,
+						    (tires < 0.0) ?
+						    sqlite3_column_double(ppStmtOneRecord,7)
+						    : tires) )
+	      || ( SQLITE_OK != sqlite3_bind_text(ppStmtUpdateRecord, 11,
+						  (notes == NULL) ?
+						  sqlite3_column_text(ppStmtOneRecord,8) 
+						  : notes,
+						  -1, SQLITE_STATIC) )
+	      || SQLITE_OK != sqlite3_bind_int64(ppStmtUpdateRecord, 12,
+						 id)
+	      || SQLITE_DONE != sqlite3_step(ppStmtUpdateRecord)) {
+	    PDEBUG("Problem saving result");
+	  }
+	  else {
+	    ret = id;
+	  }
+	  sqlite3_reset(ppStmtOneRecord);
+
+	}
+	sqlite3_reset(ppStmtUpdateRecord);
+      }
+  }
+
+  return ret;
+}
+
+/**
+ * \fn int db_delete_record(sqlite_int64 id)
+ * \brief Deletes a record
+ * \param id the id of the record
+ * \return nonzero if succesful, 0 if not
+ *
+ */
+int db_delete_record(sqlite_int64 id)
+{
+  int ret;
+
+  ret = SQLITE_ERROR;
+  if (SQLITE_OK != sqlite3_bind_int(ppStmtDeleteRecord, 1, id)
+      ||SQLITE_DONE != (ret=sqlite3_step(ppStmtDeleteRecord))) {
+    PDEBUG("Problem deleting the record");
+  }
+  sqlite3_reset(ppStmtDeleteRecord);
+
+  return ret;
+}
 
 /**
  * \fn void fuelpad_exit(void)
