@@ -25,8 +25,6 @@
 
 #include <sqlite3.h>
 
-#define ACCBUFSIZE 100
-
 //#define DEBUG
 #ifdef DEBUG
 #if (__GNUC__ > 2) && ((__GNUC__ > 3) || (__GNUC_MINOR__ > 2))
@@ -60,6 +58,13 @@
 #define PDEBUG(fmt...)
 #endif
 
+enum dbtimespan {
+  OVERALL,
+  LASTYEAR,
+  LASTMONTH,
+  SPANEND
+};
+
 extern int debug_level;
 extern volatile int ui_started;
 extern unsigned int currentdriver;
@@ -96,23 +101,27 @@ extern sqlite3_stmt *ppStmtUpdateEvent;
 double atof_l(const char *nptr, const char *locale);
 float db_get_fueltype(int carid);
 float get_emission_per_litre(int fueltype);
-float calc_co2_emission(float fill, float trip, int fueltype);
+float calc_co2_emission(float consum, int carid);
 void db_setfilename(char *fn);
 char *db_getfilename(void);
 void db_connect(void);
 void db_close(void);
 int db_open(void);
 float db_get_last_km(void);
-float db_get_last_refill(void);
+float db_get_last_refill(double newkm);
+void db_find_prev_full(double km, double *fullfill, double *fullkm);
+sqlite_int64 db_find_next_full(double km, double *fullfill, double *fullkm);
 int db_get_monthly_data(int year, double **x, double **fill, double **trip, double **consum, double **priceperlitre);
-float db_get_totalkm(void);
-float db_get_totalfill(void);
+float db_get_totalkm(enum dbtimespan timespan);
+float db_get_totalfillkm(enum dbtimespan timespan);
+float db_get_totalfill(enum dbtimespan timespan);
 int db_get_yearly_consum(int **year, int **numfills, float **minconsum, float **avgconsum, float **maxconsum);
 int db_get_consum_bin(int year, float minconsum, float maxconsum);
 float db_get_totalcost(void);
 sqlite_int64 db_get_last_insert_rowid(void);
 void db_get_driver(char **driver, int id);
 void db_get_current_driver(char **driver);
+sqlite_int64 db_get_driverid(int index);
 int db_get_numof_drivers(void);
 int db_get_numof_records(int id);
 int db_get_numof_events(int id);
@@ -124,6 +133,7 @@ int db_get_numof_alarmtypes(int id);
 int db_delete_driver(int id);
 int db_delete_car(int id);
 void db_get_current_car(char **car);
+sqlite_int64 db_get_carid(int index);
 int db_estimate_date(float km, char **estdate, char *lastdate, double interval);
 float db_estimate_km(char *lastdate, double interval);
 void db_get_last_event(int alarmid, char **day, double *km);
@@ -135,6 +145,36 @@ int db_events_exist(int alarmid);
 void db_calc_date(char **newdate, char *date, double num, char *unit);
 int db_delete_events(int alarmid);
 int db_delete_reminder(int id);
+int db_init_report_query(unsigned int carid);
+void db_reset_report_query(void);
+int db_step_report_query(void);
+const unsigned char *db_report_column_text(int col);
+double db_report_column_double(int col);
+sqlite_int64 db_add_record(char * date,
+			   double km,
+			   double trip,
+			   double fill,
+			   double consum,
+			   double price,
+			   double priceperlitre,
+			   double service,
+			   double oil,
+			   double tires,
+			   char *notes);
+sqlite_int64 db_update_record(sqlite_int64 id,
+			      char * date,
+			      double km,
+			      double trip,
+			      double fill,
+			      double consum,
+			      double price,
+			      double priceperlitre,
+			      double service,
+			      double oil,
+			      double tires,
+			      char *notes);
+int db_delete_record(sqlite_int64 id);
+void db_close_report_query(void);
 void fuelpad_exit(void);
 
 #endif
