@@ -23,14 +23,25 @@
 #include "uireport.h"
 
 /* Statistics window related*/
+#if MAEMO_VERSION_MAJOR == 5
 #define X1NOFULL_FP .15
-#define Y1NOFULL_FP .1
+#define Y1NOFULL_FP .20
+#define X2NOFULL_FP .85
+#define Y2NOFULL_FP .875
+#define XNOFULL_NEXT .95
+#define YNOFULL_NEXT .95
+#define XNOFULL_PREV .05
+#define YNOFULL_PREV .95
+#else
+#define X1NOFULL_FP .15
+#define Y1NOFULL_FP .125
 #define X2NOFULL_FP .85
 #define Y2NOFULL_FP .75
 #define XNOFULL_NEXT .95
 #define YNOFULL_NEXT .95
 #define XNOFULL_PREV .05
 #define YNOFULL_PREV .95
+#endif
 
 enum analysis {CURRENT, CONSUM, FILL, TRIP, PRICEPERLITRE, CONSUMDISTR};
 
@@ -45,6 +56,7 @@ void cb_stat( GtkAction * action, AppUIData *pui );
 void cb_trip( GtkAction * action, AppUIData *pui );
 void cb_priceperlitre( GtkAction * action, AppUIData *pui );
 void cb_consumdistr( GtkAction * action, AppUIData *pui );
+void callback_select_stat( GtkAction * action, AppUIData *pui );
 
 /**
  * \fn gboolean delete_statwin( GtkWidget *widget, GdkEvent  *event, gpointer   data )
@@ -87,7 +99,7 @@ plot_monthly_data(AppUIData *pui, enum analysis method)
 {
   static GtkPlotData *dataset;
 
-  char yearbuf[YEARBUFLEN];
+  GString *yearbuf;
   GdkColor color;
   static double *px2=NULL;
   static double *fill=NULL;
@@ -117,6 +129,8 @@ plot_monthly_data(AppUIData *pui, enum analysis method)
 
   static enum analysis currentmethod=0;
 
+  yearbuf=g_string_new(NULL);
+
   if (method != CURRENT) {
     currentmethod = method;
     plot_set_titles(pui, method);
@@ -294,8 +308,9 @@ plot_monthly_data(AppUIData *pui, enum analysis method)
   gtk_plot_data_set_line_attributes(dataset,
 				    GTK_PLOT_LINE_NONE,
 				    0, 0, 1, &color);
-  g_snprintf(yearbuf,YEARBUFLEN,"%s %d",_("Year"),currentyear);
-  gtk_plot_data_set_legend(dataset, yearbuf);
+  g_string_printf(yearbuf,"%d",currentyear);
+  gtk_plot_data_set_legend(dataset, yearbuf->str);
+  g_string_free(yearbuf, TRUE);
 
   switch (currentmethod) {
   case CONSUM:
@@ -342,6 +357,24 @@ void browse_this_year(AppUIData *pui)
   plot_monthly_data(pui, CURRENT);
 }
 
+#if MAEMO_VERSION_MAJOR == 5
+void callback_horizontal_movement(HildonPannableArea *hildonpannable,
+				  gint                direction,
+				  gdouble             initial_x,
+				  gdouble             initial_y,
+				  gpointer            data)
+{
+  switch (direction) {
+  case HILDON_MOVEMENT_LEFT:
+    browse_next_year(data);
+    break;
+  case HILDON_MOVEMENT_RIGHT:
+    browse_prev_year(data);
+    break;
+  }
+}
+#endif
+
 void callback_next_year( GtkAction * action, gpointer data )
 {
   browse_next_year(data);
@@ -383,8 +416,11 @@ void stat_window_fullscreen(gpointer data)
 void cb_stat_fullscreen( GtkAction * action, gpointer data )
 {
   stat_window_fullscreen(data);
+#if MAEMO_VERSION_MAJOR == 5
+#else
   gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(((AppUIData *)(data))->stat_menu_item_fullscreen),
 				 ((AppUIData *)(data))->statfullscreen);
+#endif
 }
 
 #if PLAINGTK == 0
@@ -393,10 +429,13 @@ gboolean stat_key_press_cb(GtkWidget * widget, GdkEventKey * event,
 {
   switch (event->keyval) {
 
+#if MAEMO_VERSION_MAJOR == 5
+#else
     case GDK_F6:
       gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(pui->stat_menu_item_fullscreen),
 				     !pui->statfullscreen);
       return TRUE;
+#endif
     case GDK_Left:
       browse_prev_year(pui);
       return TRUE;
@@ -518,12 +557,32 @@ static void create_stat_toolbar(HildonWindow *window, AppUIData *pui)
 #endif
 
 }
+
+#if MAEMO_VERSION_MAJOR == 5
+GtkWidget *create_stat_combo(void)
+{
+  GtkWidget *combo;
+
+  combo = hildon_touch_selector_new_text ();
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (combo),_("Monthly consumption"));
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (combo),_("Monthly driving distance"));
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (combo),_("Monthly average fuel price"));
+  hildon_touch_selector_append_text(HILDON_TOUCH_SELECTOR (combo),_("Yearly fill distribution"));
+  hildon_touch_selector_set_active(HILDON_TOUCH_SELECTOR (combo), 0, 0);
+
+  return combo;
+}
+#endif
+
 #if PLAINGTK == 1
 void create_statwin_menu(GtkWidget *window, GtkWidget *root, AppUIData *pui)
 #else
   void create_statwin_menu(HildonWindow *window, AppUIData *pui)
 #endif
 {
+#if MAEMO_VERSION_MAJOR == 5
+  GtkWidget *stat_combo;
+#endif
   GtkWidget *main_menu;
   GtkWidget *menu_view;
   GtkWidget *menu_analysis;
@@ -540,7 +599,58 @@ void create_statwin_menu(GtkWidget *window, GtkWidget *root, AppUIData *pui)
   GtkWidget *menubar;
   GtkWidget *item_file;
 #endif
- 
+
+#if MAEMO_VERSION_MAJOR == 5
+  GtkWidget * button;
+  
+  main_menu = HILDON_APP_MENU (hildon_app_menu_new ());
+
+/*   item_analysis = hildon_button_new_with_text(HILDON_SIZE_AUTO, */
+/* 					      HILDON_BUTTON_ARRANGEMENT_VERTICAL, */
+/* 					      _("Statistics"), */
+/* 					      NULL); */
+  item_analysis = hildon_picker_button_new(HILDON_SIZE_AUTO,
+					   HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+  hildon_button_set_title (HILDON_BUTTON(item_analysis),_("Statistics"));
+  pui->sm_stat_combo = create_stat_combo();
+  hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (item_analysis),
+                                     HILDON_TOUCH_SELECTOR (pui->sm_stat_combo));
+  hildon_app_menu_append (main_menu, GTK_BUTTON (item_analysis));
+
+  item_save = hildon_button_new_with_text(HILDON_SIZE_AUTO,
+					  HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+					  _("Save"),
+					  NULL);
+  hildon_app_menu_append (main_menu, GTK_BUTTON (item_save));
+
+  item_this = hildon_button_new_with_text(HILDON_SIZE_AUTO,
+					  HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+					  _("Show current year"),
+					  NULL);
+  hildon_app_menu_append (main_menu, GTK_BUTTON (item_this));
+
+  item_exit = hildon_button_new_with_text(HILDON_SIZE_AUTO,
+					  HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+					  _("Close"),
+					  NULL);
+  hildon_app_menu_append (main_menu, GTK_BUTTON (item_exit));
+
+  g_signal_connect(G_OBJECT(item_analysis), "value-changed",
+		   G_CALLBACK(callback_select_stat), pui);
+  g_signal_connect(G_OBJECT(item_save), "clicked",
+		   G_CALLBACK(callback_save_stat), pui);
+  g_signal_connect(G_OBJECT(item_this), "clicked",
+		   G_CALLBACK(callback_this_year), pui);
+  g_signal_connect(G_OBJECT(item_prev), "clicked",
+		   G_CALLBACK(callback_prev_year), pui);
+  g_signal_connect(G_OBJECT(item_next), "clicked",
+		   G_CALLBACK(callback_next_year), pui);
+  g_signal_connect_swapped(G_OBJECT(item_exit), "clicked",
+			   G_CALLBACK(delete_statwin), G_OBJECT(window));
+
+  hildon_window_set_app_menu(HILDON_WINDOW(window), main_menu);
+
+#else 
   main_menu = gtk_menu_new();
   menu_view = gtk_menu_new();
   menu_analysis = gtk_menu_new();
@@ -549,7 +659,7 @@ void create_statwin_menu(GtkWidget *window, GtkWidget *root, AppUIData *pui)
   item_analysis = gtk_menu_item_new_with_label(_("Statistics"));
   item_save = gtk_menu_item_new_with_label(_("Save figure..."));
   item_view = gtk_menu_item_new_with_label(_("View"));
-#if PLAINGTK == 0
+#if LIBHELP == 1
   item_help = gtk_menu_item_new_with_label(_("Help"));
 #endif
   item_exit = gtk_menu_item_new_with_label(_("Close window"));
@@ -577,7 +687,7 @@ void create_statwin_menu(GtkWidget *window, GtkWidget *root, AppUIData *pui)
   gtk_menu_append(main_menu, item_save);
   gtk_menu_append(main_menu, item_view);
   gtk_menu_append(main_menu, gtk_separator_menu_item_new());
-#if PLAINGTK == 0
+#if LIBHELP == 1
   gtk_menu_append(main_menu, item_help);
 #endif
   gtk_menu_append(main_menu, item_exit);
@@ -620,7 +730,7 @@ void create_statwin_menu(GtkWidget *window, GtkWidget *root, AppUIData *pui)
 			   G_CALLBACK(cb_consumdistr), pui);
   g_signal_connect(G_OBJECT(item_save), "activate",
 		   G_CALLBACK(callback_save_stat), pui);
-#if PLAINGTK == 0
+#if LIBHELP == 1
   g_signal_connect(G_OBJECT(item_help), "activate",
 		   G_CALLBACK(help_activated), HELP_ID_STATISTICS);
 #endif
@@ -636,13 +746,33 @@ void create_statwin_menu(GtkWidget *window, GtkWidget *root, AppUIData *pui)
   g_signal_connect(G_OBJECT(item_next), "activate",
 		   G_CALLBACK(callback_next_year), pui);
 
+#endif
+
   /* Make all menu widgets visible */
   gtk_widget_show_all(GTK_WIDGET(main_menu));
 }
 
 void plot_set_titles( AppUIData *pui, enum analysis method)
 {
+  GString *subtitle;
+
+  subtitle=g_string_new(NULL);
   gtk_plot_canvas_remove_child(GTK_PLOT_CANVAS(pui->canvas), pui->maintitle);
+  gtk_plot_canvas_remove_child(GTK_PLOT_CANVAS(pui->canvas), pui->subtitle);
+
+  if (SQLITE_OK == sqlite3_bind_int(ppStmtCurCar, 1, currentcar)) {
+    if (SQLITE_ROW == sqlite3_step(ppStmtCurCar)) {
+      g_string_printf(subtitle, "%s %s", 
+		      sqlite3_column_text(ppStmtCurCar, 0),
+		      sqlite3_column_text(ppStmtCurCar, 1));
+      g_string_append_printf(subtitle, " %s", sqlite3_column_text(ppStmtCurCar, 3));
+    }
+    sqlite3_reset(ppStmtCurCar);
+  }
+  pui->subtitle = gtk_plot_canvas_text_new("Helvetica-Bold", 12, 0, NULL, NULL, TRUE,
+					    GTK_JUSTIFY_CENTER,
+					    subtitle->str);
+  g_string_free(subtitle, TRUE);
 
   switch (method) {
   case CONSUM:
@@ -725,12 +855,17 @@ void plot_set_titles( AppUIData *pui, enum analysis method)
 
 
   gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(pui->canvas), pui->maintitle, .50, YSCALE(.050), .0, .0);
+  gtk_plot_canvas_put_child(GTK_PLOT_CANVAS(pui->canvas), pui->subtitle, .50, YSCALE(.100), .0, .0);
 
 }
 
 void create_statwin( AppUIData *pui, enum analysis method)
 {
   GtkPlotCanvasChild *fuelplot;
+  GtkWidget *hbox;
+#if MAEMO_VERSION_MAJOR == 5
+  GtkWidget *button;
+#endif
   GtkWidget *vbox1;
   GdkColor color;
   gint page_width, page_height;
@@ -744,7 +879,11 @@ void create_statwin( AppUIData *pui, enum analysis method)
   }
 
 /*   if (reportpaper == A4PAPER) { */
+#if MAEMO_VERSION_MAJOR == 5
     scale = 720.0/GTK_PLOT_A4_W;
+#else
+    scale = 720.0/GTK_PLOT_A4_W;
+#endif
     page_width = GTK_PLOT_A4_W;
     page_height = GTK_PLOT_A4_H;
 /*   } */
@@ -759,8 +898,12 @@ void create_statwin( AppUIData *pui, enum analysis method)
   gtk_window_set_title(GTK_WINDOW(pui->app->statwindow), "Plot");
   gtk_widget_set_size_request(pui->app->statwindow,800,400);
   gtk_container_border_width(GTK_CONTAINER(pui->app->statwindow),0);
+#else
+#if MAEMO_VERSION_MAJOR == 5
+  pui->app->statwindow = hildon_stackable_window_new();
 #else
   pui->app->statwindow = HILDON_WINDOW(hildon_window_new());
+#endif
   hildon_program_add_window(pui->app->program, pui->app->statwindow);
 
   /* Callback for hardware keys */
@@ -771,20 +914,55 @@ void create_statwin( AppUIData *pui, enum analysis method)
   g_signal_connect(G_OBJECT(pui->app->statwindow), "destroy",
 		   G_CALLBACK(destroy_statwin), pui);
 
+  hbox=gtk_hbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(pui->app->statwindow),hbox);
+
+#if MAEMO_VERSION_MAJOR == 5
+  button = gtk_button_new_with_label("<");
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+  g_signal_connect(G_OBJECT(button), "clicked",
+		   G_CALLBACK(callback_prev_year), pui);
+  gtk_widget_show(button);
+#endif
+
   vbox1=gtk_vbox_new(FALSE,0);
-  gtk_container_add(GTK_CONTAINER(pui->app->statwindow),vbox1);
+/*   gtk_container_add(GTK_CONTAINER(pui->app->statwindow),vbox1); */
+  gtk_container_add(GTK_CONTAINER(hbox),vbox1);
   gtk_widget_show(vbox1);
 
+#if MAEMO_VERSION_MAJOR == 5
+  pui->statscrollwin = hildon_pannable_area_new();
+  g_object_set(G_OBJECT(pui->statscrollwin), "mov-mode", 0, NULL);
+  g_object_set(G_OBJECT(pui->statscrollwin), "vscrollbar-policy", GTK_POLICY_NEVER, NULL);
+  g_object_set(G_OBJECT(pui->statscrollwin), "hscrollbar-policy", GTK_POLICY_NEVER, NULL);
+  g_signal_connect(G_OBJECT(pui->statscrollwin), "horizontal-movement",
+		   G_CALLBACK(callback_horizontal_movement), pui);
+#else
   pui->statscrollwin=gtk_scrolled_window_new(NULL, NULL);
   gtk_container_border_width(GTK_CONTAINER(pui->statscrollwin),0);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(pui->statscrollwin),
 				  GTK_POLICY_AUTOMATIC,GTK_POLICY_NEVER);
-   gtk_box_pack_start(GTK_BOX(vbox1),pui->statscrollwin, TRUE, TRUE,0);
-   gtk_widget_show(pui->statscrollwin);
+#endif
+  gtk_box_pack_start(GTK_BOX(vbox1),pui->statscrollwin, TRUE, TRUE,0);
+  gtk_widget_show(pui->statscrollwin);
+
+#if MAEMO_VERSION_MAJOR == 5
+  button = gtk_button_new_with_label(">");
+  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+  g_signal_connect(G_OBJECT(button), "clicked",
+		   G_CALLBACK(callback_next_year), pui);
+  gtk_widget_show(button);
+#endif
+
+  gtk_widget_show(hbox);
 
   pui->canvas = gtk_plot_canvas_new(page_width, page_height, scale);
   gtk_widget_set_usize(pui->canvas,page_width,page_height);
+#if MAEMO_VERSION_MAJOR == 5
+  hildon_pannable_area_add_with_viewport(HILDON_PANNABLE_AREA(pui->statscrollwin), pui->canvas);
+#else
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pui->statscrollwin), pui->canvas);
+#endif
 
   gdk_color_parse("light blue", &color);
 /*   gdk_color_alloc(gtk_widget_get_colormap(pui->canvas), &color); */
@@ -799,10 +977,13 @@ void create_statwin( AppUIData *pui, enum analysis method)
   create_statwin_menu(pui->app->statwindow, pui);
 #endif
 
+#if MAEMO_VERSION_MAJOR == 5
+#else
 #if PLAINGTK == 1
   create_stat_toolbar(pui->app->statwindow, vbox1, pui);
 #else
   create_stat_toolbar(pui->app->statwindow, pui);
+#endif
 #endif
 
   pui->active_plot = new_layer(pui);
@@ -837,7 +1018,7 @@ void create_statwin( AppUIData *pui, enum analysis method)
 
   /* (GtkPlot *plot, GtkPlotBorderStyle border, gint shadow_width);*/
   gtk_plot_set_legends_border(GTK_PLOT(pui->active_plot), 2, 0);
-  gtk_plot_legends_move(GTK_PLOT(pui->active_plot), .9, YSCALE(.05));
+  gtk_plot_legends_move(GTK_PLOT(pui->active_plot), .95, YSCALE(.05));
   /*  gtk_plot_hide_legends(GTK_PLOT(pui->active_plot)); */
 
   gtk_plot_axis_set_title(gtk_plot_get_axis(GTK_PLOT(pui->active_plot), GTK_PLOT_AXIS_BOTTOM), _("Month"));
@@ -862,9 +1043,77 @@ void create_statwin( AppUIData *pui, enum analysis method)
 
   gtk_widget_show_all(GTK_WIDGET(pui->app->statwindow));
 
+#if MAEMO_VERSION_MAJOR == 5
+#else
   if (pui->mainfullscreen) 
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(pui->stat_menu_item_fullscreen),
 				   !pui->statfullscreen);
+#endif
+}
+
+void callback_quick_stat( GtkAction * action, AppUIData *pui )
+{
+  GtkWidget *dialog;
+  GtkWidget *vbox;
+  GtkWidget *label;
+  GtkWidget *separator;
+  gint i;
+  GString *str;
+  float totalkm;
+  float totalfillkm;
+  float totalfill;
+  gchar *heading[] = {gettext_noop("Overall"),
+		      gettext_noop("Last year"),
+		      gettext_noop("Last month")};
+
+  dialog = gtk_dialog_new_with_buttons(_("Quick statistics"),
+				       GTK_WINDOW(pui->app->mainwindow),
+				       GTK_DIALOG_MODAL,
+				       GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+				       NULL);
+  vbox = GTK_WIDGET(GTK_DIALOG(dialog)->vbox);
+
+  str=g_string_new(NULL);
+
+  for (i=0;i<sizeof(heading)/sizeof(char *);i++) {
+    label = gtk_label_new(gettext(heading[i]));
+    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
+    gtk_widget_show(label);
+
+    separator = gtk_hseparator_new();
+    gtk_box_pack_start(GTK_BOX(vbox), separator, TRUE, TRUE, 0);
+    gtk_widget_show(separator);
+
+    totalkm=db_get_totalkm(i);
+    totalfillkm=db_get_totalfillkm(i);
+    totalfill=db_get_totalfill(i);
+    g_string_printf(str, "%.0f ", SIlength2user(totalkm));
+    if (curunit.lengthunit==SI)
+      g_string_append_printf(str, _("km"));
+    else
+      g_string_append_printf(str, _("miles"));
+
+    g_string_append_printf(str, ": ");
+    if (totalfillkm != 0.0)
+      g_string_append_printf(str, "%.1f ",SIconsumption2user(totalfill/totalfillkm*100));
+    else
+      g_string_append_printf(str, "%.1f ",0.0);
+    if (curunit.consumeunit==SI)
+      g_string_append_printf(str, _("l/100 km"));
+    else
+      g_string_append_printf(str, _("MPG"));
+
+    label = gtk_label_new(str->str);
+    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
+    gtk_widget_show(label);
+
+  }
+
+  gtk_dialog_run(GTK_DIALOG(dialog));
+  gtk_widget_destroy(GTK_WIDGET(dialog));
+
+  g_string_free(str, TRUE);
+
 }
 
 void callback_stat( GtkAction * action, AppUIData *pui )
@@ -911,3 +1160,28 @@ void cb_consumdistr( GtkAction * action, AppUIData *pui )
 {
   plot_monthly_data(pui, CONSUMDISTR);
 }
+
+#if MAEMO_VERSION_MAJOR == 5
+void callback_select_stat( GtkAction * action, AppUIData *pui )
+{
+  gint analtype;
+
+  analtype = hildon_touch_selector_get_active(HILDON_TOUCH_SELECTOR(pui->sm_stat_combo), 0);
+  switch (analtype) {
+  case 0:
+    plot_monthly_data(pui, CONSUM);
+    break;
+  case 1:
+    plot_monthly_data(pui, TRIP);
+    break;
+  case 2:
+    plot_monthly_data(pui, PRICEPERLITRE);
+    break;
+  case 3:
+    plot_monthly_data(pui, CONSUMDISTR);
+    break;
+  default:
+    g_print("No corresponding analysis found\n");
+  }
+}
+#endif
