@@ -27,10 +27,16 @@
  *
  */
 
-void convdate(char *s, size_t max, const char *outformat, 
+#ifndef _UTIL_H_
+#define _UTIL_H_
+
+void convdate(char **s, const char *outformat, 
 	      const char *informat, const char *date);
-void date2sqlite(char *s, size_t max, const char *format, const char *date);
-void gettimefmt(char *aika, const char *format);
-void gettime(char *aika);
+void date2sqlite(char **s, const char *format, const char *date);
+void gettimefmt(char **aika, const char *format);
+void gettime(char **aika);
 int getyear(void);
-void getyearmonthday(char *datestr, int *year, int *month, int *day);
+void getyearmonthday(const char *datestr, int *year, int *month, int *day);
+double doubleornothing(double input);
+
+#endif
